

//	(C) 2017 R-MIK Sawomir Skrzyski
//	slawomir.skrzynski@ep.com.pl


//#define	USART_BITS		8			// 5..9 (FTDI 7..9)
//#define	USART_PARITY	'n'			// 'n', 'e', 'o'
//#define	USART_STOP		1			// 1, 2
//#define	USART_IRQRX
//#define	USART_RXBUF	16			// ! Potega dwjki
//#define	USART_IRQTX
//#define	USART_TXBUF	16			// ! Potega dwjki


#include <rmik/usart.h>


//=======================================================================//
//				Obsuga RS (UART / USART)							//
//------------------------------------------------------------------//
void Usart_Init()
{
byte ursel=0;


	#ifdef __AVR_ATmega162__
	  ursel = (1<<URSEL);
	  UCSRC &= ~(1<<URSEL);
	#endif
	UBRRH = UBRRH_VALUE;				// Set baud (wybrane przez BAUD)
	UBRRL = UBRRL_VALUE;

	#if USART_BITS==5
	  UCSRC = (0<<UCSZ0) | ursel;		// Set frame format: 5n1
	#elif USART_BITS==6
	  UCSRC = (1<<UCSZ0) | ursel;		// Set frame format: 6n1
	#elif USART_BITS==7
	  UCSRC = (2<<UCSZ0) | ursel;		// Set frame format: 7n1
	#elif USART_BITS==8 || USART_BITS==9
	  UCSRC = (3<<UCSZ0) | ursel;		// Set frame format: 8n1 lub 9n1
	#else
	  #error "Zla liczba bitow"
	#endif
	#if USART_STOP==1
	  //UCSRC |= (0<<USBS) | ursel; 	// xx1
	#elif USART_STOP==2
	  UCSRC |= (1<<USBS) | ursel; 	// xx2
	#else
	  #error "Zla liczba bitow stopu"
	#endif
	#if USART_PARITY=='n'
	  //UCSR0C |=(0<<UPM0);				// xNx
	#elif USART_PARITY=='e'
	  UCSRC |=(2<<UPM);				// xNx
	#elif USART_PARITY=='o'
	  UCSRC |=(3<<UPM);				// xNx
	#else
	  #error "Zla parzystosc"
	#endif
	UCSRB = (1<<RXEN) | (1<<TXEN)	// Enable receiver and transmitter
			#if USART_BITS==9
			  | (1<<UCSZ02)				// Set frame format: 9n1 (UCSZ=7)
			#endif
			#ifdef USART_IRQRX
			  | (1<<RXCIE)				// Waczenie przerwa odborczych
			#endif
			#ifdef USART_IRQTX
			  | (1<<TXCIE)
			#endif
			;

	#if USE_2X
	 UCSRA |= _BV(U2X);
	#endif
}


//------------------------------------------------------------------//
//				Sprawdzenie czy jest znak w RS						//
//------------------------------------------------------------------//
char Usart_Test()
{
	#ifdef USART_IRQRX
	  if ( Usart_PtrRxR != Usart_PtrRxW ) return( TRUE );
	#else
	  if ( (UCSRA & (1<<RXC)) ){ return( TRUE ); }		// Jeli odernao znak to wyjd z funkcji
	#endif

	return( FALSE );									// Nie ma znaku
}


//------------------------------------------------------------------//
//				Czekanie na znak z RS 								//
//------------------------------------------------------------------//
word Usart_Receive()
{
char znak;

	wdt_reset();
	if ( (UCSRA & (1<<RXC)) )									// Jeli odernao znak to odczytaj go i wyjd z funkcji
		{
		if ( (UCSRA & (1<UPE)) ){ return( PARITYERR ); }		// Sprawdzenie parzystoci
		if ( (UCSRA & (1<FE)) ) { return( FRAMEERR ); }		// Sprawdzenie bdu ramki (np zy znak stopu)
		if ( (UCSRA & (1<DOR)) ){ return( OVERRUNERR ); }		// Sprawdzenie przepenienia bufora
		znak = UDR;
		if ( UCSRB & RXB8 ) znak |= 0x100;
		return( znak );
		}

	return ( NONE );			// Min timeout, wyjd z bdem
}


//------------------------------------------------------------------//
//				Wysanie znaku po RS								//
//------------------------------------------------------------------//
void Usart_Transmit( word data )
{
	/* Wait for empty transmit buffer */
	while ( !( UCSRA & (1<<UDRE)) ){ ; }

	#if USART_BITS==9
	  if ( data & 0x100 ) { UCSRB |= (1<<TXB8); } else { UCSRB &= ~(1<<TXB8); }
	#endif
	/* Put data into buffer, sends the data */
	UDR = data;
}


//------------------------------------------------------------------//
//				Czyszczenie FIFO UARTA								//
//------------------------------------------------------------------//
void Usart_Flush()
{
	#ifdef USART_IRQRX
	  Usart_PtrRxR = Usart_PtrRxW = 0;
	#else
	  byte dummy;
	  while ( UCSRA & (1<<RXC) ) dummy = UDR;
	#endif
}


//----------------------------------------------------------------------//
///* USART, Rx Complete */
///* USART RX complete */
//#define USART_RX_vect			_VECTOR(28)
//#define SIG_Usart_RECV			_VECTOR(28)

///* USART Data register Empty */
///* USART Data Register Empty */
//#define USART_UDRE_vect			_VECTOR(29)
//#define SIG_USART_DATA			_VECTOR(29)
//----------------------------------------------------------------------//
#ifdef USART_IRQRX
//#define	SIG_USART_RECV		USART_RX_vect
SIGNAL(USART_RX_vect)			// Musi byc ISR lub SIGNAL a nie INTERUPT
{
word bit8=0;

#warning "zaimplementowac sprawdzanie FIFO"
// petla do...while( UCSR0A & (1<<RXC0) )
// to samo dla usart 1


	#if USART_BITS==9
	  if ( UCSRB & (1<<RXB8) ) bit8 = 0x100;
	#endif
	Usart_RxBuf[Usart_PtrRxW++] = UDR | bit8;	// wpisz znak do bufora i zwiksz dugo
	Usart_PtrRxW &= USART_RXBUF-1;					// Ograniczenie wieloci bufora (max 255 bajtw)
	if ( Usart_PtrRxW == Usart_PtrRxR )			// Jeli wskanik zapisu "dogoni" wskanik odczytu
		{											// to przepeniono bufor RS-a
//		PrintString_P( PSTR(ERR_BUF_USB_TXT""CRLF) );
		}
}
#endif


//----------------------------------------------------------------------//
word Usart_Get()
{
	#ifdef USART_IRQRX
	  if ( Usart_PtrRxR == Usart_PtrRxW ) return( NONE );
	  return( Usart_RxBuf[ Usart_PtrRxR ++ ] );
	#else
	  return( Usart_Receive() );
	#endif
}


//----------------------------------------------------------------------//
byte Usart_Put( word c )
{
	#ifdef USART_IRQTX
//todo: jeli jest miejsce
	if ( Usart_PtrTxR )

	  Usart_TxBuf[ Usart_PtrTxW++ ] = c;
	  Usart_PtrTxW &= DEF_TXBUF-1
//todo: zrobi na IRQ napenianie bufora nadawczego, jak nie mona to zwr FALSE
// Jak rozwiza problem, e znak jest pierwszy?
//	Moe na zasadzie znanikw?

	#else
	  Usart_Transmit( c ); return( TRUE );
	#endif
}


//----------------------------------------------------------------------//
#ifdef USART_IRQTX
SIGNAL(USART_UDRE_vect)			// Musi byc ISR lub SIGNAL a nie INTERUPT
{
	#if USART_BITS==9
	  if ( Usart_TxBuf[ Usart_PtrTxR] & 0x100 ) UCRSB |= (1<<TXRB8); else UCRSB &= ~(1<<TXRB8);
	#endif
	UDR = Usart_TxBuf[ Usart_PtrTxR++ ];
#error	ograniczenie zakresu
}
#endif


//----------------------------------------------------------------------//
